/** FunnyMu: portable creatiVision emulator ******************/
/**                                                         **/
/**                      funnysdl.c                         **/
/**                                                         **/
/** This file contains implementation for the creatiVision  **/
/** specific hardware: VDP, etc. Initialization code and    **/
/** definitions needed for the machine-dependent drivers    **/
/** are also here.                                          **/
/**                                                         **/
/** Unofficial version by Luca "MADrigal" Antignano, 2007   **/
/**                                                         **/
/** Copyright (C) Paul Hayter 2001                          **/
/**     based on ColEm by Marat Fayzullin                   **/
/**                                                         **/
/** Copyright (C) Marat Fayzullin 1994-1998                 **/
/**     You are not allowed to distribute this software     **/
/**     commercially. Please, notify me, if you make any    **/
/**     changes to this file.                               **/
/*************************************************************/

#include "SDL/SDL.h"		/* All SDL App's need this   */
#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include "funny.h"
#include "funnysdl.h"

/* Globals */
#define PLAYBUF_SIZE 1024
static Uint8 playbuf[PLAYBUF_SIZE];
static volatile int snd_read=0;
static SDL_AudioSpec spec;

int SaveCPU =1;			/* 1 = freeze when focus out */
int UseSHM  =1;			/* 1 = use MITSHM            */
int UseSound=1;			/* 1 = use sound             */

SDL_Surface *sdl_screen;
Uint32 sdl_colours[16];
char sdl_title[]="FunnyMu v0.49 (unofficial by MADrigal)";

#define WIDTH  256
#define HEIGHT 192

extern char VideoMode[256];	/* read from INI file (windowed/fullscreen mode */





/** sdl_setpalette() *****************************************/
/** This sets colours palette for the TMS9928 video chip    **/
/*************************************************************/
void sdl_setpalette(void) {
  struct { Uint8 R,G,B; } Palette[16] = {
    /* palette from MAME by S. Young */
    {0x00, 0x00, 0x00},	// 0 transparent (should be set on VDP as bg color)
    {0x00, 0x00, 0x00},	// 1 black
    {0x21, 0xc8, 0x42},	// 2 medium green
    {0x5e, 0xdc, 0x78}, // 3 light green
    {0x54, 0x55, 0xed},	// 4 dark blue
    {0x7d, 0x76, 0xfc},	// 5 light blue
    {0xd4, 0x52, 0x4d},	// 6 dark red
    {0x42, 0xeb, 0xf5},	// 7 cyan
    {0xfc, 0x55, 0x54},	// 8 medium red
    {0xff, 0x79, 0x78},	// 9 light red
    {0xd4, 0xC1, 0x54},	// A dark yellow
    {0xe6, 0xCe, 0x80},	// B light yellow
    {0x21, 0xb0, 0x3b},	// C dark green
    {0xc9, 0x5b, 0xba},	// D magenta
    {0xcc, 0xcc, 0xcc},	// E gray
    {0xff, 0xff, 0xff}	// F white

/*   Original palette from ColEm
    {0x00, 0x00, 0x00},
    {0x00, 0x00, 0x00},
    {0x20, 0xC0, 0x20},
    {0x60, 0xE0, 0x60},
    {0x20, 0x20, 0xE0},
    {0x40, 0x60, 0xE0},
    {0xA0, 0x20, 0x20},
    {0x40, 0xC0, 0xE0},
    {0xE0, 0x20, 0x20},
    {0xE0, 0x60, 0x60},
    {0xC0, 0xC0, 0x20},
    {0xC0, 0xC0, 0x80},
    {0x20, 0x80, 0x20},
    {0xC0, 0x40, 0xA0},
    {0xA0, 0xA0, 0xA0},
    {0xE0, 0xE0, 0xE0}
*/
  };
  int i;

  printf("SDL is setting palette...");
  for (i = 0; i < 16; i++) {
    sdl_colours[i] = SDL_MapRGB(sdl_screen->format, Palette[i].R, Palette[i].G, Palette[i].B);
    sdl_colours[i] += sdl_colours[i] << 16;
  }
  printf("OK\n");
}





/** sdl_snd_callback() ***************************************/
/** This os called when the audio device is ready for more  **/
/** data by the SDL_AudoSpec structure                      **/
/*************************************************************/
static void sdl_snd_callback(void *udata, Uint8 *stream, int len) {
  int i;
  for (i = 0; i < len; ++i) {
    if (snd_read == PLAYBUF_SIZE) {
      SN76496Update(0, playbuf, PLAYBUF_SIZE / 2);
      snd_read = 0;
    }
#ifndef LSB_FIRST
    if (i & 1) {
      stream[i - 1] = (playbuf[snd_read++]);
    }
    else {
      stream[i + 1] = (playbuf[snd_read++]);
    }
#else
    stream[i] = (playbuf[snd_read++] - 0x6000) >> 1;
#endif
    }
}





/** sound_init() *********************************************/
/** This initializes audio emulation device via SDL         **/
/*************************************************************/
int sound_init(void) {
  SDL_AudioSpec req;

  req.freq     = 22050;
  req.format   = AUDIO_S16;
  req.channels = 1;
  req.samples  = 256;
  req.callback = sdl_snd_callback;
  req.userdata = NULL;

  if (SDL_OpenAudio(&req, &spec) < 0) {
    fprintf(stderr, "SDL is opening audio stream...error: %s!\n", SDL_GetError());
    return 0;
  }
  else {
    printf("SDL is opening audio stream...OK\n");
  }

  if (spec.channels != 1) {
    fprintf(stderr, "SDL is opening audio stream...couldn't get mono audio format!\n");
    SDL_CloseAudio();
  }

  printf("SDL is initializing audio...");

  SN76496Init(0, VDP_CLOCK / 15, 0, spec.freq);

  printf("OK\n");

  SDL_PauseAudio(0);
  return(0);
}





/** init_sdl() ***********************************************/
/** This initializes video and audio emulation devices via  **/
/** SDL                                                     **/
/*************************************************************/
int init_sdl(void) {
  int resx = 512;
  int resy = 384;
  int bpp  = 16;

  /* Initialize defaults, Video and Audio */
  if ((SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO) == -1)) { 
    printf("SDL is initializing...: %s.\n", SDL_GetError());
    exit(-1);
  }

  atexit(SDL_Quit);

  printf("SDL is initializing...OK\n");
  sound_init();

  if (strcmp(VideoMode, "windowed") == 0) {
    sdl_screen = SDL_SetVideoMode(resx, resy, bpp, SDL_HWSURFACE);			// sets video as windowed
    printf("Video mode: windowed %dx%dx%d...OK\n", resx, resy, bpp);
  }
  else
    if (strcmp(VideoMode, "fullscreen")==0) {
      sdl_screen = SDL_SetVideoMode(resx, resy, bpp, (SDL_HWSURFACE, SDL_FULLSCREEN));	// sets video as fullscreen
      printf("Video mode: fullscreen...OK\n");
    }

  if (sdl_screen == NULL) {
    fprintf(stderr, "SDL couldn't set %dx%dx%d video mode: %s\n", resx, resy, bpp, SDL_GetError());
    return(1);
  }

  sdl_setpalette();
  SDL_WM_SetCaption(sdl_title, sdl_title);

  return(0);
}





/** sdl_getticks() *******************************************/
/** Get the number of milliseconds since the SDL library    **/
/** initialization.                                         **/
/*************************************************************/
Uint32 sdl_getticks(void) {
  return(SDL_GetTicks());
}





/** OnBreak() ************************************************/
/** This function is called on signals                      **/
/*************************************************************/
static void OnBreak(int Arg) {
  ExitNow = 1;
  signal(Arg, OnBreak);
}





/** InitMachine() ********************************************/
/** Allocate resources needed by Unix/X-dependent code.     **/
/*************************************************************/
int InitMachine(void) {
  init_sdl();

  /* Catch all signals */
#ifndef WIN32
  signal(SIGHUP,  OnBreak);
  signal(SIGINT,  OnBreak);
  signal(SIGQUIT, OnBreak);
  signal(SIGTERM, OnBreak);
#endif

  return(1);
}





/** TrashMachine() *******************************************/
/** Deallocate all resources taken by InitMachine().        **/
/*************************************************************/
void TrashMachine(void) {
  unsigned long L;
  int J;

  if (Verbose)
    printf("Shutting down emulation...\n");
}





/** ScreenCapture() ******************************************/
/** Saves current screen data onto file.                    **/
/*************************************************************/
void ScreenCapture(void) {
  int i = 1;
  char fn[15];
  FILE *fp;

  while (i <= 999) {
    sprintf(fn, "capture%03d.bmp", i);
    fp=fopen(fn, "r");
    if (!fp) {
      SDL_SaveBMP(sdl_screen, fn);
      i = 999;
    }
    else
      fclose(fp);
    i++;
  } 
}





/** SaveListing() ********************************************/
/** Saves current VRAM data from $1C00 to $3FFF to plain    **/
/** text file (used to save BASIC listings)                 **/
/**                                                         **/
/** Adapted from Giovanni Ortu's CvEmu2 source code         **/
/*************************************************************/
void SaveListing(void) {
  #define START_CHECK 0x1800
  #define START_VID 0x1c00
  #define END_VID 0x4000
  #define MEM_SIZE (END_VID - START_VID)
  #define USE_ORIGINAL_LINE 0				// 0 to remove any zero's before line number	(e.g. 005 REM -> 5    REM)	
							// 1 to keep zero's 				(e.g. 005 REM -> 005  REM)
  int exit=0;
  int allowed_count=0;
  int exit_line=0;
  int found=0;
  int counter=0;

  char buff[10000][64];
  memset(buff, 0x00, 10000*64);

  char allowed[10000];
  memset(allowed, 0x00, 10000);

  while (!exit) {
    int memline = VRAM[START_CHECK + allowed_count * 2] * 256 + VRAM[START_CHECK + allowed_count * 2 + 1];

    if ((memline > 0xff00) || (START_CHECK + allowed_count * 2 > 0x1bff))
      exit = 1;
    else {
      allowed[memline] = 1;
      allowed_count++;
    }
  }

  if (allowed_count != 0) {
    exit = 0;

    while (!exit) {
      if (counter>MEM_SIZE
          || (VRAM[START_VID + counter+0] == 0
          && VRAM[START_VID + counter+1] == 0
          && VRAM[START_VID + counter+2] == 0
          && VRAM[START_VID + counter+3] == 0)
        )
        exit = 1;
      else
      {
        exit_line = 0;
        char temp[64], linest[7], line[64];
        memset(temp, 0x00, 64);

        sprintf(linest, "%c%c%c%c\0",
          VRAM[START_VID + counter + 0],
          VRAM[START_VID + counter + 1],
          VRAM[START_VID + counter + 2],
          VRAM[START_VID + counter + 3]
        );

        int linenum = atoi(linest);
        counter += 5;
        int index = 0;

        while (counter < MEM_SIZE && !exit_line) {
          temp[index] = VRAM[START_VID + counter];
          if (VRAM[START_VID + counter] == 0x0d) {
            temp[index + 1] = 0x0a;
            temp[index + 2] = 0x00;
            index++;
            exit_line = 1;
            found = 1;
          }
          counter++;
          index++;
        }

        if (USE_ORIGINAL_LINE)
          sprintf(line, "%s %s", linest, temp);
        else
          sprintf(line, "%-4d %s", linenum, temp);
        strcpy(buff[linenum], line);
      }
    }
  }

  char fn[12];
  FILE *fl, *fs;

  if (found) {						// if any valid listing is found in VRAM
    int i = 1;							// finds unexisting savexxx.txt file to save listing
    while (i <= 999) {
      sprintf(fn, "save%03d.txt", i);
      if (fl = fopen(fn, "r")) {
        i++;
        fclose(fl);
      }
      else { 							// new filename generated, proceed to saving listing
        fs = fopen(fn, "wb");
        int j = 0;
        for (j = 0; j < 9999; j++)
          if (buff[j][0] != 0x00 && allowed[j] != 0)		// save existing data if present in line counter buffer
            fprintf(fs, "%s", buff[j]);
        fclose(fs);
        i = 1000;
      }
    } 
  }
}





/** LoadListing() ********************************************/
/** Loads "load.txt" data into VRAM from $1C00 to $3FFF     **/
/** (used to load BASIC listings)                           **/
/*************************************************************/
void LoadListing(void) {
  #define START_CHECK 0x1800
  #define START_VID 0x1c00
  #define END_VID   0x4000
  #define SECOND_CHECK 0x1400
  #define MEM_SIZE (END_VID - START_VID)

  char line[64];
  char temp[64], temp2[64];
  int count=0;

  FILE *fp;
  fp=fopen("load.txt", "r");
  if (fp) {
    char buff[10000][64];
    memset(buff, 0x00, 10000 * 64);

    while (!feof(fp) && fgets(line, 64, fp)) {
      int start = 0;
      memset(temp, ' ', 64);
      int len = strlen(line);
      int num_line = -1;
      
      while (start < len && line[start] != ' ')		// parsing string to find line number
        temp[start] = line[start++];

      if (start > 0) {					// ok, line number found
        num_line = atoi(temp);					// this is the line number
        while (start < len && line[start] == ' ')
         start++;						// skip white spaces after line number

        if (num_line > 0 && num_line < 10000)			// start -> index of the first 'non space' char	
          memcpy(buff[num_line], &line[start], len - start + 1);	// inserting only allowed line numbers

        count++;					// line counter;
      }
    }

    if (count != 0) {					// at least one line found
      int i = 0;
      int offset_mem = 0;
      int offset_check = 0;
      memset(&VRAM[START_CHECK], 0xff, START_VID-START_CHECK);
      memset(&VRAM[START_VID], 0x00, END_VID-START_VID);

      int prevlen = 0xdc00;
      for (i = 0; i < 10000; i++) {
        if (buff[i][0] != '\0')	{			// the current line (i) exists
          sprintf(line, "%-4u %s", i, buff[i] );		// formatting line
          int cc = 0;
          int len2 = strlen(line);
            
          while (cc < len2 && line[cc] != 0x0d && line[cc] != 0x0a)	// copy formatted line into VDP memory
            VRAM[START_VID + offset_mem++] = line[cc++];

          VRAM[START_VID + offset_mem++] = 0x0d;		// adding end of line

          unsigned char high = prevlen / 256;
          unsigned char low = (unsigned char)(((float)prevlen / 256 - high) * 256);

          VRAM[SECOND_CHECK + offset_check * 2] = high;
          VRAM[SECOND_CHECK + offset_check * 2+1] = low;

          prevlen += cc + 1;

          high = i / 256;

          low=(unsigned char)(((float)i / 256 - high) * 256);

          VRAM[START_CHECK + offset_check * 2] = high;
          VRAM[START_CHECK + offset_check * 2 + 1] = low;

          offset_check++;

          printf("%s",line);
        }
      }
    }

    fclose(fp);
  }
}





/** Joysticks()   ********************************************/
/** Manages keypresses into emulated keyboard and joysticks **/
/** (plus the function keys for emulator purposes)          **/
/*************************************************************/
void Joysticks(void) {
  SDL_Event sdl_event;
  if (SDL_PollEvent(&sdl_event)) {
    if (sdl_event.type == SDL_KEYDOWN) {
      switch(sdl_event.key.keysym.sym) {

#ifdef DEBUG
        case SDLK_F1:		CPU.Trace = !CPU.Trace; break;
#endif
        case SDLK_F2:		LogSnd = !LogSnd; break;
        case SDLK_F3:		Reset6502(&CPU); break;
	case SDLK_F5:		SaveListing(); break;
	case SDLK_F7:		LoadListing(); break;
	case SDLK_F12:		ScreenCapture(); break;
        case SDLK_ESCAPE:	ExitNow = 1; break;

// PIA 1 DOWN
        case SDLK_1:		KEYTBL[14] &= 0xf3; break;	// 1/!
        case SDLK_2:		KEYTBL[13] &= 0xcf; break;	// 2/"
        case SDLK_3:		KEYTBL[13] &= 0x9f; break;	// 3/#
        case SDLK_4:		KEYTBL[13] &= 0xd7; break;	// 4/$
        case SDLK_5:		KEYTBL[13] &= 0xb7; break;	// 5/%
        case SDLK_6:		KEYTBL[13] &= 0xaf; break;	// 6/&

        case SDLK_LCTRL:	KEYTBL[14] &= 0x7f; break;	// CNTL		P1 right button
        case SDLK_RCTRL:	KEYTBL[14] &= 0x7f; break;	// CNTL		P1 right button
        case SDLK_q:		KEYTBL[13] &= 0xe7; break;	// Q
        case SDLK_w:		KEYTBL[13] &= 0xf3; break;	// W
        case SDLK_e:		KEYTBL[13] &= 0xeb; break;	// E
        case SDLK_r:		KEYTBL[13] &= 0xdb; break;	// R
        case SDLK_t:		KEYTBL[13] &= 0xbb; break;	// T

        case SDLK_BACKSPACE:	KEYTBL[13] &= 0xf6; break;	// BACKSPACE
        case SDLK_a:		KEYTBL[13] &= 0xee; break;	// A
        case SDLK_s:		KEYTBL[13] &= 0xde; break;	// S
        case SDLK_d:		KEYTBL[13] &= 0xbe; break;	// D
        case SDLK_f:		KEYTBL[13] &= 0xfc; break;	// F
        case SDLK_g:		KEYTBL[13] &= 0xfa; break;	// G

        case SDLK_LSHIFT:	KEYTBL[13] &= 0x7f; break;	// SHIFT	P1 left button
        case SDLK_RSHIFT:	KEYTBL[13] &= 0x7f; break;	// SHIFT	P1 left button
        case SDLK_z:		KEYTBL[13] &= 0xf5; break;	// Z
        case SDLK_x:		KEYTBL[13] &= 0xed; break;	// X
        case SDLK_c:		KEYTBL[13] &= 0xdd; break;	// C
        case SDLK_v:		KEYTBL[13] &= 0xbd; break;	// V
        case SDLK_b:		KEYTBL[13] &= 0xf9; break;	// B

        case SDLK_DOWN:		KEYTBL[14] &= 0xfd; break;	// 		P1 down
        case SDLK_UP:		KEYTBL[14] &= 0xf7; break;	// 		P1 up
        case SDLK_LEFT:		KEYTBL[14] &= 0xdf; break;	// 		P1 left
        case SDLK_RIGHT:	KEYTBL[14] &= 0xfb; break;	// 		P1 right

// PIA 2 DOWN
        case SDLK_7:		KEYTBL[7] &= 0xf9; break;	// 7/'
        case SDLK_8:		KEYTBL[7] &= 0xbd; break;	// 8/(
        case SDLK_9:		KEYTBL[7] &= 0xdd; break;	// 9/)
        case SDLK_0:		KEYTBL[7] &= 0xed; break;	// 0/@
        case SDLK_MINUS:	KEYTBL[7] &= 0xf5; break;	// :/*
        case SDLK_EQUALS:	KEYTBL[7] &= 0x7f; break;	// -/=		P2 left button
        case SDLK_INSERT:	KEYTBL[7] &= 0x7f; break;	// -/=		P2 left button

        case SDLK_y:		KEYTBL[7] &= 0xfa; break;	// Y
        case SDLK_u:		KEYTBL[7] &= 0xfc; break;	// U
        case SDLK_i:		KEYTBL[7] &= 0xbe; break;	// I
        case SDLK_o:		KEYTBL[7] &= 0xde; break;	// O
        case SDLK_p:		KEYTBL[7] &= 0xee; break;	// P
        case SDLK_RETURN:	KEYTBL[7] &= 0xf6; break;	// RET'N

        case SDLK_h:		KEYTBL[7] &= 0xbb; break;	// H
        case SDLK_j:		KEYTBL[7] &= 0xdb; break;	// J
        case SDLK_k:		KEYTBL[7] &= 0xeb; break;	// K
        case SDLK_l:		KEYTBL[7] &= 0xf3; break;	// L
        case SDLK_SEMICOLON:	KEYTBL[7] &= 0xe7; break;	// ;/+
        case SDLK_TAB:		KEYTBL[11] &= 0x7f; break;	// FORWARD	P2 right button
        case SDLK_PAGEUP:	KEYTBL[11] &= 0x7f; break;	// FORWARD	P2 right button

        case SDLK_n:		KEYTBL[7] &= 0xaf; break;	// N
        case SDLK_m:		KEYTBL[7] &= 0xb7; break;	// M
        case SDLK_COMMA:	KEYTBL[7] &= 0xd7; break;	// ,/<
        case SDLK_PERIOD:	KEYTBL[7] &= 0x9f; break;	// ./>
        case SDLK_SLASH:	KEYTBL[7] &= 0xcf; break;	// //?
        case SDLK_SPACE:	KEYTBL[11] &= 0xf3; break;	// SPACE

        case SDLK_END:		KEYTBL[11] &= 0xfd; break;	// 		P2 down
        case SDLK_HOME:		KEYTBL[11] &= 0xf7; break;	// 		P2 up
        case SDLK_DELETE:	KEYTBL[11] &= 0xdf; break;	// 		P2 left
        case SDLK_PAGEDOWN:	KEYTBL[11] &= 0xfb; break;	// 		P2 right
      }
    }

    if (sdl_event.type == SDL_KEYUP) {
      switch(sdl_event.key.keysym.sym) {
// PIA 1 UP
        case SDLK_1:		KEYTBL[14] |= 0x0c; break;	// 1/!
        case SDLK_2:		KEYTBL[13] |= 0x30; break;	// 2/"
        case SDLK_3:		KEYTBL[13] |= 0x60; break;	// 3/#
        case SDLK_4:		KEYTBL[13] |= 0x28; break;	// 4/$
        case SDLK_5:		KEYTBL[13] |= 0x48; break;	// 5/%
        case SDLK_6:		KEYTBL[13] |= 0x50; break;	// 6/&

        case SDLK_LCTRL:	KEYTBL[14] |= 0x80; break;	// CNTL		P1 right button
        case SDLK_RCTRL:	KEYTBL[14] |= 0x80; break;	// CNTL		P1 right button
        case SDLK_q:		KEYTBL[13] |= 0x18; break;	// Q
        case SDLK_w:		KEYTBL[13] |= 0x0c; break;	// W
        case SDLK_e:		KEYTBL[13] |= 0x14; break;	// E
        case SDLK_r:		KEYTBL[13] |= 0x24; break;	// R
        case SDLK_t:		KEYTBL[13] |= 0x44; break;	// T

        case SDLK_BACKSPACE:	KEYTBL[13] |= 0x09; break;	// BACKSPACE
        case SDLK_a:		KEYTBL[13] |= 0x11; break;	// A
        case SDLK_s:		KEYTBL[13] |= 0x21; break;	// S
        case SDLK_d:		KEYTBL[13] |= 0x41; break;	// D
        case SDLK_f:		KEYTBL[13] |= 0x03; break;	// F
        case SDLK_g:		KEYTBL[13] |= 0x05; break;	// G

        case SDLK_LSHIFT:	KEYTBL[13] |= 0x80; break;	// SHIFT	P1 left button
        case SDLK_RSHIFT:	KEYTBL[13] |= 0x80; break;	// SHIFT	P1 left button
        case SDLK_z:		KEYTBL[13] |= 0x0a; break;	// Z
        case SDLK_x:		KEYTBL[13] |= 0x12; break;	// X
        case SDLK_c:		KEYTBL[13] |= 0x22; break;	// C
        case SDLK_v:		KEYTBL[13] |= 0x42; break;	// V
        case SDLK_b:		KEYTBL[13] |= 0x06; break;	// B

        case SDLK_DOWN:		KEYTBL[14] |= 0x02; break;	// 		P1 down
        case SDLK_UP:		KEYTBL[14] |= 0x08; break;	// 		P1 up
        case SDLK_LEFT:		KEYTBL[14] |= 0x20; break;	// 		P1 left
        case SDLK_RIGHT:	KEYTBL[14] |= 0x04; break;	// 		P1 right

// PIA 2 UP
        case SDLK_7:		KEYTBL[7] |= 0x06; break;	// 7/'
        case SDLK_8:		KEYTBL[7] |= 0x42; break;	// 8/(
        case SDLK_9:		KEYTBL[7] |= 0x22; break;	// 9/)
        case SDLK_0:		KEYTBL[7] |= 0x12; break;	// 0/@
        case SDLK_MINUS:	KEYTBL[7] |= 0x0a; break;	// :/*
        case SDLK_EQUALS:	KEYTBL[7] |= 0x80; break;	// -/=		P2 left button
        case SDLK_INSERT:	KEYTBL[7] |= 0x80; break;	// -/=		P2 left button

        case SDLK_y:		KEYTBL[7] |= 0x05; break;	// Y
        case SDLK_u:		KEYTBL[7] |= 0x03; break;	// U
        case SDLK_i:		KEYTBL[7] |= 0x41; break;	// I
        case SDLK_o:		KEYTBL[7] |= 0x21; break;	// O
        case SDLK_p:		KEYTBL[7] |= 0x11; break;	// P
        case SDLK_RETURN:	KEYTBL[7] |= 0x09; break;	// RET'N

        case SDLK_h:		KEYTBL[7] |= 0x44; break;	// H
        case SDLK_j:		KEYTBL[7] |= 0x24; break;	// J
        case SDLK_k:		KEYTBL[7] |= 0x14; break;	// K
        case SDLK_l:		KEYTBL[7] |= 0x0c; break;	// L
        case SDLK_SEMICOLON:	KEYTBL[7] |= 0x18; break;	// ;/+
        case SDLK_TAB:		KEYTBL[11] |= 0x80; break;	// FORWARD	P2 right button
        case SDLK_PAGEUP:	KEYTBL[11] |= 0x80; break;	// FORWARD	P2 right button

        case SDLK_n:		KEYTBL[7] |= 0x50; break;	// N
        case SDLK_m:		KEYTBL[7] |= 0x48; break;	// M
        case SDLK_COMMA:	KEYTBL[7] |= 0x28; break;	// ,/<
        case SDLK_PERIOD:	KEYTBL[7] |= 0x60; break;	// ./>
        case SDLK_SLASH:	KEYTBL[7] |= 0x30; break;	// //?
        case SDLK_SPACE:	KEYTBL[11] |= 0x0c; break;	// SPACE

        case SDLK_END:		KEYTBL[11] |= 0x02; break;	// 		P2 down
        case SDLK_HOME:		KEYTBL[11] |= 0x08; break;	// 		P2 up
        case SDLK_DELETE:	KEYTBL[11] |= 0x20; break;	// 		P2 left
        case SDLK_PAGEDOWN:	KEYTBL[11] |= 0x04; break;	// 		P2 right
      }
    }

    if (sdl_event.type == SDL_QUIT) {
       ExitNow = 1;
    }
  }
}

#include "Common.h"
